<?php

/*
=====================================================
 File: pi.scripturizer.php
-----------------------------------------------------
 Purpose: Scripturizer plugin
=====================================================

*/

$plugin_info = array(
						'pi_name'			=> 'Scripturizer',
						'pi_version'		=> '1.0',
						'pi_author'			=> 'Scott Yang (conv. Paul Burdick)',
						'pi_author_url'		=> 'http://scott.yang.id.au/project/phpscripturizer/',
						'pi_description'	=> 'Link scripture references to ESV at Good News Publisher or Bible Gateway.',
						'pi_usage'			=> Scripturizer::usage()
					);
					

define('DEFAULT_SCRIPTURIZER_VERSION', 'ESV');

/**
 * Namespace for Scripturizer.
 */
class Scripturizer {

	var $return_data = '';

	//-------------------------------------
    //  Constructor
    //-------------------------------------
	function Scripturizer()
	{
		global $TMPL;
		
		$this->return_data = $this->scripturize($TMPL->tagdata);
	
		return $this->return_data;
	}
	
	    
	// ----------------------------------------
	//  Plugin Usage
	// ----------------------------------------

	function usage()
	{
	ob_start(); 
	?>
	
{exp:scripturizer}

John 3:16

{/exp:scripturizer}

	<?php
	$buffer = ob_get_contents();
	
	ob_end_clean(); 

	return $buffer;
	}
	// END


    /**
     * Hyperlink scripture reference in text.
     *
     * @param data String containing the text.
     * @param callback A function object to be used to replace bible
     * references to hyperlinks.
     * @return Scripturized text.
     */
    function addLink($data, $callback) {
        $volumes = 'I+|1st|2nd|3rd|First|Second|Third|1|2|3';
        $books   = 'Genesis|Gen\.?|Exodus|Exod?|Ex\.?|Leviticus|Lev\.?|'.
            'Levit?\.?|Numbers|Nmb\.?|Numb?\.?|Deuteronomy|Deut?\.?|Dt\.?|'.
            'Joshua|Josh?\.?|Judges|Jdg\.?|Judg?\.?|Ruth|Ru\.?|Samuel|Sam\.?|'.
            'Sml\.?|Kings|Kn?gs?\.?|Kin?\.?|Chronicles|Chr\.?|Chron\.?|Ezra|'.
            'Ez\.?|Nehemiah|Nehem?\.?|Neh\.?|Esther|Esth?\.?|Job|Jb\.?|'.
            'Psalms?|Psa?\.?|Proverbs?|Prov?\.?|Ecclesiastes|Eccl?\.?|'.
            'Songs? of Solomon|Song?\.?|Songs|Isaiah|Isa\.?|Jeremiah|Jer\.?|'.
            'Jerem\.?|Lamentations|Lam\.?|Lament?\.?|Ezekiel|Ezek?\.?|Daniel|'.
            'Dan\.?|Hosea|Hos\.?|Joel|Jo\.|Amos|Am\.?|Obadiah|Obad?\.?|Jonah|'.
            'Jon\.|Micah|Mic\.?|Nahum|Nah\.?|Habakkuk|Hab\.?|Habak|Zephaniah|'.
            'Zeph\.?|Haggai|Hagg?\.?|Zechariah|Zech?\.?|Malachi|Malac?\.?|'.
            'Mal\.?|Mat{1,2}hew|Matt?\.?|Mark|Mr?k\.?|Luke|Lu?k\.?|John|'.
            'Jh?n\.?|Jo\.|Acts?|Ac\.?|Romans|Rom\.?|Corinthians|Cor\.?|'.
            'Corin\.?|Galatians|Gal\.?|Galat\.?|Ephesians|Eph\.?|Ephes\.?|'.
            'Phil+ippians|Phili?\.?|Colossians|Col\.?|Colos\.?|Thessalonians|'.
            'Thes?\.?|Timothy|Tim\.?|Titus|Tts\.?|Tit\.?|Philemon|Phil?\.?|'.
            'Hebrews|Hebr?\.?|James|Jam\.?|Jms\.?|Peter|Pete?\.?|Jude|Ju\.?|'.
            'Revelations?|Rev\.?|Revel\.?';

        $verses  = '\d+(:\d+)?(?:\s?[-&]\s?\d+)?';
        $version = '\[([A-Za-z\-]+)\]';

        return preg_replace_callback("/
            (?:($volumes)\s+)?
            ($books)
            \s+
            ($verses(?:\s?,\s?$verses)*)
            (?:$version)?
            /mx",
            $callback, $data);
    }
    
    /**
     * See if the requested version is valid
     *
     * Listed below are the versions that BibleGateway supports
     *
     * @param value from the custom field
     * @return boolean
     */
    function version_exists($filter) {
        switch ($filter) {
            case "ESV":
            case "KJV":
            case "NIV":
            case "MSG":
            case "NASB":
            case "AMP":
            case "NLT":
            case "HCSB":
            case "CEV":
            case "NKJV":
            case "NLV":
            case "KJ21":
            case "ASV":
            case "WE":
            case "YLT":
            case "DARBY":
            case "WYC":
            case "NIRV":
            case "NIV-UK":
                return true;
            default:
                return false;
        }
    }
    
    /**
     * Per-post filtering support. This is added as a filter hook
     * on 'the_content', so it gets called during The Loop for
     * each post displayed. We check for a post custom field
     * named 'scripture_version', and if valid, set the version
     * based on the value there.
     *
     * Logic borrowed from TFS Core (Text Filter Suite) by Dougal Campbell
     * http://dougal.gunters.org/blog/2004/08/30/text-filter-suite
     *
     * @return version
     */
     function getVersion() {
         $version = DEFAULT_SCRIPTURIZER_VERSION;
         if (function_exists('get_post_custom_values')) {
             $filters = get_post_custom_values('scripture_version');
             if (is_array($filters)) {
                 foreach ($filters as $filter) {
                     $filter = trim($filter);
                     $filter = strtoupper($filter);
                     if (Scripturizer::version_exists($filter)) {
                         $version = $filter;
                         break;
                     }
                 }
            }
         }

         return $version;
     }

     /**
      * Return a tuple of (URL, title) from a version string (ESV, NIV, etc).
      *
      * @param version Name of the version of the Bible
      * @return A tuple containing (URL, title) correspond to this version.
      */
     function getVersionInfo($version) {
        if (preg_match('/esv/i', $version)) {
            $title = 'English Standard Version Bible';
            $link  = 'http://www.gnpcb.org/esv/search/?go=Go&amp;q=';
        } else {
            $title = 'Bible Gateway';
            $link  = "http://biblegateway.com/cgi-bin/bible?language=english".
                     "&amp;version=$version&amp;passage=";
        }

        return array($link, $title);
     }

    /**
     * Callback function for replacing scripture references in the text to
     * HTML hyperlinks.
     *
     * @param match Regex matching result array.
     * @param link URL to prepend the reference.
     * @param title Text to be used in title=".." attribute of the link.
     */
    function addLinkCallback($match, $link, $title) {
        global $_scripture_link;

        // Check whether Bible translation version has been overriden for this
        // link.
		$version = ( !isset($match['6'])) ? FALSE : $match['6'];
		
        if ($version) {
            list($link, $title) = Scripturizer::getVersionInfo($version);
        }
        
        $vol = $match[1];
        $bok = $match[2];
        $ver = $match[3];

        $psg = htmlentities(($vol ? "$vol " : "").$bok.($ver ? " $ver" : ""));
        $ver = preg_replace(array('/:/', '/[,&;]/', '/\s*/'),
                            array('%3A', '%2C',     ''),
                            $ver);

        // Trim off the dot at the end of book before putting it on the link.
        $bok = preg_replace('/\.$/', '', $bok);
        $link .= ($vol ? "$vol+" : "").($bok ? "$bok+" : "").$ver;
        return "<a href=\"$link\" title=\"$title\">$psg</a>";
    }

    /**
     * Scripturize a text string.
     *
     * @param text Original text to be scripturized.
     * @param version Translation of the scripture.
     * @return Scripturized text.
     */
    function scripturize($text, $version=0) {
        // If 'version' has been set to false, then we will return the text
        // without scripturizing it. If 'version' is not a string, and we are
        // inside the WordPress environment ("get_post_custom_values" is
        // defined), then we will try to resolve the Bible version from the
        // post variable. (Thanks to Jon Mark for patching it)
        if ($version === false) {
            return $text;
        } elseif (!is_string($version)) {
            $version = Scripturizer::getVersion();
        }

        // Pre-calculate the scripture link URL and title.
        list($link, $title) = Scripturizer::getVersionInfo($version);

        $callback = create_function('$match', "return Scripturizer::addLinkCallback(".'$match'.", '$link', '$title');");
        $text = preg_split("/(<a href.*?<\/a>|<code.*?<\/code>|<pre.*?<\/pre>|<.+?".">)/ms", $text, -1, PREG_SPLIT_DELIM_CAPTURE);

        $newtext = array();
        foreach ($text as $chunk) {
            if (!preg_match('/<.*>/', $chunk)) {
                $chunk = Scripturizer::addLink($chunk, $callback);
            }
            $newtext[] = $chunk;
        }
        return implode('', $newtext);
    }
}
// END Scripturizer Class

?>